import pandas as pd
from aistac.handlers.abstract_handlers import AbstractSourceHandler, ConnectorContract, AbstractPersistHandler, \
    HandlerFactory

__author__ = 'Johan Gielstra'


class RedisSourceHandler(AbstractSourceHandler):
    """ A mongoDB source handler"""

    def __init__(self, connector_contract: ConnectorContract):
        """ initialise the Hander passing the source_contract dictionary """
        # required module import
        self.redis = HandlerFactory.get_module('redis')
        super().__init__(connector_contract)
        self._file_state = 0
        self._changed_flag = True

    def supported_types(self) -> list:
        """ The source types supported with this module"""
        return ['redis']

    def load_canonical(self, **kwargs) -> dict:
        """ returns the canonical dataset based on the source contract
            The canonical in this instance is a dictionary that has the headers as the key and then
            the ordered list of values for that header
        """
        conn = None
        if not isinstance(self.connector_contract, ConnectorContract):
            raise ValueError("The Connector Contract is not valid")
        # this supports redis hmap only...
        cc_params = self.connector_contract.kwargs
        cc_params.update(kwargs)     # Update with any passed though the call

        match = cc_params.get('match', '*')
        count = cc_params.get('count', 1000)
        keys = cc_params.get('keys')
        if not keys or len(keys) == 0:
            raise ValueError("RedisConnector requires an array of 'keys'")
        try:
            conn = self.redis.from_url(self.connector_contract.uri)
            rtn_dict = {'id': []}
            for rowkey in conn.scan_iter(match, count):
                """
                {
                    "col1" = [1,2,3],
                    "col2" = ["a","b","c"]
                }
                """
                rowdata = conn.hgetall(rowkey)
                rtn_dict.get('id').append(rowkey.decode())
                for colkey in keys:
                    if colkey not in rtn_dict:
                        rtn_dict[colkey] = []
                    raw_col_val = rowdata.get(str.encode(colkey))
                    if raw_col_val:
                        col_val = raw_col_val.decode()
                    else:
                        col_val = None
                    rtn_dict.get(colkey).append(col_val)
            conn.close()
            return rtn_dict
        except Exception as error:
            print(error)
        finally:
            if conn is not None:
                conn.close()
                print('Database connection closed.')

    def exists(self) -> bool:
        return True

    def has_changed(self) -> bool:
        """ returns if the file has been modified"""
        # TODO: Add in change logic here
        state = None
        if state != self._file_state:
            self._changed_flag = True
            self._file_state = state
        return self._changed_flag

    def reset_changed(self, changed: bool = False):
        """ manual reset to say the file has been seen. This is automatically called if the file is loaded"""
        changed = changed if isinstance(changed, bool) else False
        self._changed_flag = changed


class RedisPersistHandler(RedisSourceHandler, AbstractPersistHandler):
    def persist_canonical(self, canonical: pd.DataFrame, **kwargs) -> bool:
        """ persists the canonical dataset """
        if not isinstance(self.connector_contract, ConnectorContract):
            return False
        return self.backup_canonical(canonical=canonical, uri=self.connector_contract.uri, **kwargs)

    def remove_canonical(self) -> bool:
        if not isinstance(self.connector_contract, ConnectorContract):
            return False
        _cc = self.connector_contract
        raise NotImplementedError("remove_canonical for RedisPersistHandler not yet implemented.")

    def backup_canonical(self, canonical: pd.DataFrame, uri: str, **kwargs) -> bool:
        """ creates a backup of the canonical to an alternative URI  """
        if not isinstance(self.connector_contract, ConnectorContract):
            return False
        _cc = self.connector_contract
        persist_params = kwargs if isinstance(kwargs, dict) else _cc.kwargs
        persist_params.update(_cc.parse_query(uri=uri))
        hashprefix = persist_params.get('prefix')
        if hashprefix is None:
            raise ValueError("RedisPersistHandler requires a `prefix` to be provided")
        # use `ifFieldName` to upsert records otherwise assume id is the index of the record in the data frame
        id_field_name = persist_params.get('idFieldName')
        use_index_for_id = False
        to_dict_kwargs = {"orient": "records"}
        if id_field_name is None:
            use_index_for_id = True
        if hashprefix is None:
            raise ValueError("RedisPersistHandler requires a `prefix` to be provided")
        conn = self.redis.from_url(self.connector_contract.uri)
        # TODO if persist_params.get("ordered") is None:
        # ` ordered=persist_params.get("ordered")` is this needed ?
        count = 0
        records = canonical.to_dict(**to_dict_kwargs)
        for rec in records:
            if use_index_for_id:
                idx = count
            else:
                idx = rec[id_field_name]
            conn.hmset(f'{hashprefix}.{idx}', rec)
            count = count+1
        return count == canonical.shape[0]



import unittest
import pandas as pd
import os
from pprint import pprint

from ds_connectors.handlers.s3_handlers import S3SourceHandler, S3PersistHandler
from aistac.handlers.abstract_handlers import ConnectorContract, HandlerFactory


class S3HandlerTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_aws_connector_init(self):
        handler = S3PersistHandler(connector_contract=ConnectorContract(uri='s3://project-hadron-cs-repo/factory/healthcare/members', module_name='', handler=''))
        data = {'a': [1,2,3,4,5]}
        handler.persist_canonical(data)
        result = handler.load_canonical()
        self.assertTrue(isinstance(result, dict))
        result = handler.load_canonical(read_params={'as_dataframe': True})
        self.assertTrue(isinstance(result, pd.DataFrame))


if __name__ == '__main__':
    unittest.main()



import pandas as pd
from aistac.handlers.abstract_handlers import AbstractSourceHandler, ConnectorContract, HandlerFactory

__author__ = 'Darryl Oatridge, Neil Pasricha'


class HiveSourceHandler(AbstractSourceHandler):
    """ A Hive source handler"""

    def __init__(self, connector_contract: ConnectorContract):
        """ initialise the Handler passing the source_contract dictionary """
        self.pyhive = HandlerFactory.get_module('pyhive.hive')
        super().__init__(connector_contract)
        self._file_state = 0
        self._changed_flag = True

    def supported_types(self) -> list:
        """ The source types supported with this module"""
        return ['hive']

    def load_canonical(self, **kwargs) -> [dict, pd.DataFrame]:
        """ returns the canonical dataset based on the source contract
            The canonical in this instance is a dictionary that has the headers as the key and then
            the ordered list of values for that header
        """
        if not isinstance(self.connector_contract, ConnectorContract):
            raise ValueError("The Connector Contract is not valid")
        database = self.connector_contract.path
        host = self.connector_contract.hostname
        user = self.connector_contract.username
        password = self.connector_contract.password
        auth = self.connector_contract.get_key_value('auth', '')
        configuration = self.connector_contract.get_key_value('configuration', '')
        kerberos_service_name = self.connector_contract.get_key_value('kerberos_service_name', '')
        thrift_transport = self.connector_contract.get_key_value('thrift_transport', '')
        canonical = self.connector_contract.get_key_value('canonical', 'dict')
        query = self.connector_contract.query
        host_name, port = host.rsplit(sep=':')
        conn = self.pyhive.hive.Connection(host=host_name, port=port, username=user, password=password,
                                           database=database, configuration=configuration, auth=auth,
                                           kerberos_service_name=kerberos_service_name,
                                           thrift_transport=thrift_transport, **kwargs)
        # return a pandas DataFrame
        if canonical.lower().endswith('pandas'):
            return pd.read_sql(query, conn)
        # default return a dictionary
        cursor = conn.cursor()
        cursor.execute(query)

        columns = [i[0] for i in cursor.description]
        rows = cursor.fetchall()
        rtn_dict = {}
        for row in rows:
            for index in range(len(row)):
                if columns[index] not in rtn_dict:
                    rtn_dict[columns[index]] = []
                rtn_dict.get(columns[index]).append(row[index])
        cursor.close()
        return rtn_dict

    def exists(self) -> bool:
        return True

    def has_changed(self) -> bool:
        """ returns if the file has been modified"""
        # TODO: Add in change logic here
        state = None
        if state != self._file_state:
            self._changed_flag = True
            self._file_state = state
        return self._changed_flag

    def reset_changed(self, changed: bool = False):
        """ manual reset to say the file has been seen. This is automatically called if the file is loaded"""
        changed = changed if isinstance(changed, bool) else False
        self._changed_flag = changed
